@@ -6,9 +6,8 @@
 
 #include <QStandardPaths>
 #include <QDir>
-#include <QNetworkReply>
-#include <QNetworkRequest>
 #include <QStringBuilder>
+#include <QGuiApplication>
 
 #include "asyncdatabase.h"
 
@@ -28,6 +27,25 @@ Library::Library(QObject *parent)
     }()))
 {
     m_database->runMigrations(":/migrations/");
+    m_searches = new SearchHistoryModel(m_database->getResults<SingleValue<QString>>("select distinct (search_query) from searches order by search_id desc"), this);
+    connect(this, &Library::changeFavourites, [this]() {
+        auto future = m_database->getResults<Song>("select * from favourites natural join songs order by favourites.rowid desc");
+        m_favourites= new FavouritesModel(std::move(future), this);
+        Q_EMIT favouritesChanged();
+    });
+    Q_EMIT changeFavourites();
+
+    connect(this, &Library::changePlaybackHistory, [this]() {
+        //playbackHistory
+        auto future = m_database->getResults<PlayedSong>("select * from played_songs natural join songs");
+        m_playbackHistory = new PlaybackHistoryModel(std::move(future), this);
+
+        //mostPlayed
+        auto future2 = m_database->getResults<PlayedSong>("select * from played_songs natural join songs order by plays desc limit 10");
+        m_mostPlayed = new PlaybackHistoryModel(std::move(future2), this);
+        Q_EMIT playbackHistoryChanged();
+    });
+    Q_EMIT changePlaybackHistory();
 }
 
 Library::~Library() = default;
@@ -40,20 +58,19 @@ Library &Library::instance()
 
 FavouritesModel *Library::favourites()
 {
-    auto future = m_database->getResults<Song>("select * from favourites natural join songs order by favourites.rowid desc");
-    return new FavouritesModel(std::move(future), this);
+    return m_favourites;
 }
 
 void Library::addFavourite(const QString &videoId, const QString &title, const QString &artist, const QString &album)
 {
     connectFuture(addSong(videoId, title, artist, album), this, [=, this] {
-        connectFuture(m_database->execute("insert or ignore into favourites (video_id) values (?)", videoId), this, &Library::favouritesChanged);
+        connectFuture(m_database->execute("insert or ignore into favourites (video_id) values (?)", videoId), this, &Library::changeFavourites);
     });
 }
 
 void Library::removeFavourite(const QString &videoId)
 {
-    connectFuture(m_database->execute("delete from favourites where video_id = ?", videoId), this, &Library::favouritesChanged);
+    connectFuture(m_database->execute("delete from favourites where video_id = ?", videoId), this, &Library::changeFavourites);
 }
 
 FavouriteWatcher *Library::favouriteWatcher(const QString &videoId)
@@ -66,37 +83,49 @@ FavouriteWatcher *Library::favouriteWatcher(const QString &videoId)
 
 SearchHistoryModel *Library::searches()
 {
-    return new SearchHistoryModel(m_database->getResults<SingleValue<QString>>("select distinct (search_query) from searches order by search_id desc"), this);
+    return m_searches;
 }
 
 void Library::addSearch(const QString &text)
 {
+    m_searches->addSearch(text);
     connectFuture(m_database->execute("insert into searches (search_query) values (?)", text), this, &Library::searchesChanged);
 }
 
-void Library::removeSearch(const QString &text)
+void Library::addTemporarySearch(const QString &text)
 {
+    m_searches->addSearch(text);
+    Q_EMIT searchesChanged();
+}
+
+void Library::removeSearch(const QString &text) {
+    m_searches->removeSearch(text);
     connectFuture(m_database->execute("delete from searches where search_query = ?", text), this, &Library::searchesChanged);
 }
 
+void Library::removeTemporarySearch(const QString& text)
+{
+    m_searches->removeSearch(text);
+    Q_EMIT searchesChanged();
+}
+
+
 PlaybackHistoryModel *Library::playbackHistory()
 {
-    auto future = m_database->getResults<PlayedSong>("select * from played_songs natural join songs");
-    return new PlaybackHistoryModel(std::move(future), this);
+    return m_playbackHistory;
 }
 
 void Library::addPlaybackHistoryItem(const QString &videoId, const QString &title, const QString &artist, const QString &album)
 {
     connectFuture(addSong(videoId, title, artist, album), this, [=, this] {
         connectFuture(m_database->execute("insert or ignore into played_songs (video_id, plays) values (?, ?)", videoId, 0), this, [=, this] {
-            connectFuture(m_database->execute("update played_songs set plays = plays + 1 where video_id = ? ", videoId), this, &Library::playbackHistoryChanged);
+            connectFuture(m_database->execute("update played_songs set plays = plays + 1 where video_id = ? ", videoId), this, &Library::changePlaybackHistory);
         });
     });
 }
-
 void Library::removePlaybackHistoryItem(const QString &videoId)
 {
-    connectFuture(m_database->execute("delete from played_songs where video_id = ?", videoId), this, &Library::playbackHistoryChanged);
+    connectFuture(m_database->execute("delete from played_songs where video_id = ?", videoId), this, &Library::changePlaybackHistory);
 }
 
 WasPlayedWatcher *Library::wasPlayedWatcher(const QString& videoId)
@@ -108,10 +137,11 @@ WasPlayedWatcher *Library::wasPlayedWatcher(const QString& videoId)
 }
 
 
+
+
 PlaybackHistoryModel *Library::mostPlayed()
 {
-    auto future = m_database->getResults<PlayedSong>("select * from played_songs natural join songs order by plays desc limit 10");
-    return new PlaybackHistoryModel(std::move(future), this);
+    return m_mostPlayed;
 }
 
 QNetworkAccessManager &Library::nam()
@@ -125,52 +155,6 @@ QFuture<void> Library::addSong(const QString &videoId, const QString &title, con
     return m_database->execute("insert or replace into songs (video_id, title, artist, album) values (?, ?, ?, ?)", videoId, title, artist, album);
 }
 
-void ThumbnailSource::setVideoId(const QString &id) {
-    if (m_videoId == id) {
-        return;
-    }
-
-    m_videoId = id;
-    Q_EMIT videoIdChanged();
-
-    const QString cacheDir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) % QDir::separator() % "thumbnails";
-    QDir(cacheDir).mkpath(QStringLiteral("."));
-    const QString cacheLocation = cacheDir % QDir::separator() % id % ".webp";
-
-    if (QFile::exists(cacheLocation)) {
-        setCachedPath(QUrl::fromLocalFile(cacheLocation));
-        return;
-    }
-
-    auto *reply = Library::instance().nam().get(QNetworkRequest(QUrl("https://i.ytimg.com/vi_webp/" % m_videoId % "/maxresdefault.webp")));
-
-    auto storeResult = [this, cacheLocation](QNetworkReply *reply) {
-        QFile file(cacheLocation);
-        file.open(QFile::WriteOnly);
-        file.write(reply->readAll());
-        setCachedPath(QUrl::fromLocalFile(cacheLocation));
-
-        reply->deleteLater();
-    };
-
-    connect(reply, &QNetworkReply::errorOccurred, this, [this, storeResult](auto error) {
-        if (error == QNetworkReply::NetworkError::ContentNotFoundError) {
-            qDebug() << "Naive thumbnail resolution failed, falling back to yt-dlp (slower)";
-
-            connectFuture(YTMusicThread::instance()->extractVideoInfo(m_videoId), this, [this, storeResult](auto info) {
-                auto *reply = Library::instance().nam().get(QNetworkRequest(QUrl(QString::fromStdString(info.thumbnail))));
-                connect(reply, &QNetworkReply::finished, this, [reply, storeResult]() {
-                    storeResult(reply);
-                });
-            });
-        }
-    });
-
-    connect(reply, &QNetworkReply::finished, this, [reply, storeResult]() {
-        storeResult(reply);
-    });
-}
-
 PlaybackHistoryModel::PlaybackHistoryModel(QFuture<std::vector<PlayedSong> > &&songs, QObject *parent)
     : QAbstractListModel(parent)
 {
@@ -217,6 +201,12 @@ QVariant PlaybackHistoryModel::data(const QModelIndex &index, int role) const {
     Q_UNREACHABLE();
 }
 
+std::vector<PlayedSong> PlaybackHistoryModel::getPlayedSong() const
+{
+    return m_playedSongs;
+}
+
+
 FavouritesModel::FavouritesModel(QFuture<std::vector<Song>> &&songs, QObject *parent)
     : QAbstractListModel(parent)
 {
@@ -260,6 +250,10 @@ QVariant FavouritesModel::data(const QModelIndex &index, int role) const {
     Q_UNREACHABLE();
 }
 
+std::vector<Song> FavouritesModel::getFavouriteSongs() const {
+    return m_favouriteSongs;
+}
+
 FavouriteWatcher::FavouriteWatcher(Library *library, const QString &videoId)
     : QObject(library), m_videoId(videoId), m_library(library)
 {
@@ -293,15 +287,38 @@ int SearchHistoryModel::rowCount(const QModelIndex &parent) const {
     return parent.isValid() ? 0 : m_history.size();
 }
 
+void SearchHistoryModel::removeSearch(const QString &search) {
+    int row = getRow(search);
+    beginRemoveRows({}, row, row);
+    m_history.erase(m_history.begin() + row);
+    endRemoveRows();
+}
+
+size_t SearchHistoryModel::getRow(const QString &search) const {
+    auto itr = find_if(m_history.begin(), m_history.end(), [&](const auto &checkedValue) {
+        return checkedValue.value == search;
+    });
+    size_t i = std::distance(m_history.begin(), itr);
+    Q_ASSERT(i < m_history.size());
+    return i;
+}
+
 QVariant SearchHistoryModel::data(const QModelIndex &index, int role) const {
     switch (role) {
-    case Qt::DisplayRole:
-        return m_history[index.row()].value;
+        case Qt::DisplayRole:
+            return m_history[index.row()].value;
     }
-
+    
     Q_UNREACHABLE();
 }
 
+void SearchHistoryModel::addSearch(const QString& search) {
+    beginInsertRows({}, 0, 0);
+    m_history.insert(m_history.begin(), {search});
+    endInsertRows();
+}
+
+
 WasPlayedWatcher::WasPlayedWatcher(Library* library, const QString& videoId)
     : QObject(library), m_videoId(videoId), m_library(library)
 {
