@@ -3,12 +3,14 @@
 // SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #pragma once
 
-#include <QAbstractListModel>
-#include <QFuture>
-
 #include <ytmusic.h>
 
 #include "abstractytmusicmodel.h"
+#include "library.h"
+
+class PlaylistModel;
+class AlbumModel;
+class LocalPlaylistModel;
 
 class UserPlaylistModel : public AbstractYTMusicModel
 {
@@ -23,7 +25,10 @@ class UserPlaylistModel : public AbstractYTMusicModel
     Q_PROPERTY(QString currentVideoId READ currentVideoId NOTIFY currentVideoIdChanged)
     Q_PROPERTY(int currentIndex READ currentIndex NOTIFY currentVideoIdChanged)
     Q_PROPERTY(bool canSkip READ canSkip NOTIFY canSkipChanged)
+    Q_PROPERTY(bool canSkipBack READ canSkipBack NOTIFY canSkipBackChanged)
     Q_PROPERTY(QString lyrics READ lyrics NOTIFY lyricsChanged)
+    Q_PROPERTY(QUrl webUrl READ webUrl NOTIFY currentVideoIdChanged)
+
 
 public:
     enum Role {
@@ -46,6 +51,7 @@ public:
     Q_SIGNAL void initialVideoIdChanged();
 
     QString nextVideoId() const;
+    QString previousVideoId() const;
 
     QString currentVideoId() const;
     void setCurrentVideoId(const QString &videoId);
@@ -54,7 +60,11 @@ public:
     int currentIndex() const;
 
     bool canSkip() const;
+    bool canSkipBack() const;
+
     Q_SIGNAL void canSkipChanged();
+    Q_SIGNAL void canSkipBackChanged();
+
 
     QString playlistId() const;
     void setPlaylistId(const QString &playlistId);
@@ -67,13 +77,27 @@ public:
     QString lyrics() const;
     Q_SIGNAL void lyricsChanged();
 
+    QUrl webUrl() const;
+
     Q_INVOKABLE void next();
+    Q_INVOKABLE void previous();
+
     Q_INVOKABLE void skipTo(const QString &videoId);
     Q_INVOKABLE void playNext(const QString &videoId, const QString &title, const std::vector<meta::Artist> &artists);
     Q_INVOKABLE void append(const QString &videoId, const QString &title, const std::vector<meta::Artist> &artists);
     Q_INVOKABLE void clear();
+    Q_INVOKABLE void clearExceptCurrent();
     Q_INVOKABLE void remove(const QString &videoId);
     Q_INVOKABLE void shufflePlaylist();
+    Q_INVOKABLE void appendPlaylist(PlaylistModel *playlistModel);
+    Q_INVOKABLE void appendAlbum(AlbumModel *albumModel);
+
+    Q_INVOKABLE void playFavourites(FavouritesModel *favouriteModel, bool shuffled);
+    Q_INVOKABLE void appendFavourites(FavouritesModel *favouriteModel, bool shuffled);
+    Q_INVOKABLE void playPlaybackHistory(PlaybackHistoryModel *playbackHistory, bool shuffled);
+    Q_INVOKABLE void appendPlaybackHistory(PlaybackHistoryModel *playbackHistory, bool shuffled);
+    Q_INVOKABLE void playLocalPlaylist(LocalPlaylistModel *playlistModel, bool shuffled);
+    Q_INVOKABLE void appendLocalPlaylist(LocalPlaylistModel *playlistModel, bool shuffled);
 
 private:
     void emitCurrentVideoChanged(const QString &oldVideoId);
